// Diaphragme, comme un chat.
// Se ferme vite si il y a trop de lumière, s'ouvre lentement si tout est sombre

uint32_t amm_absoluteMin, amm_absoluteMax;
uint32_t amm_curMax, amm_curMin;
uint32_t amm_delayRapide, amm_delaiLent;
uint32_t amm_timerLent, amm_timerRapide;
uint32_t amm_hysteresisZero;

// Typiquement : 0, 1023
void adaptativeMinMaxInit(uint32_t aAbsoluteMin, uint32_t aAbsoluteMax) {
  amm_absoluteMin = aAbsoluteMin;
  amm_absoluteMax = aAbsoluteMax;

  // début
  amm_curMax = amm_absoluteMax;
  amm_curMin = amm_absoluteMin;
  
  uint32_t rapide_s = 3, lent_s = 30; // nombre de secondes pour passer de min à max
  amm_delayRapide = (rapide_s*1000000)/(amm_absoluteMax - amm_absoluteMin);
  amm_delaiLent = (lent_s*1000000)/(amm_absoluteMax - amm_absoluteMin);
  amm_timerLent = micros();
  amm_timerRapide = micros();
  // petit hysteresis autour de zero, pour ne pas avoir de clignotements
  // on ne voit rien en dessous de 1/100eme de l'amplitude
  amm_hysteresisZero = (amm_absoluteMax - amm_absoluteMin)/100;
}

uint32_t amm_getCurMin() {
  return amm_curMin;
}
uint32_t amm_getCurMax() {
  return amm_curMax;
}


int adaptativeMap(int curVal) {
  if (micros() - amm_timerLent > amm_delaiLent) {
    // on augmmente de 1 tous les amm_delaiLent
    amm_timerLent = micros();
    if (curVal < amm_curMax) amm_curMax = amm_curMax - 1;
    if (curVal > amm_curMin) amm_curMin = amm_curMin + 1;
  }
  if (micros() - amm_timerRapide > amm_delayRapide) {
    // on augmmente de 1 tous les amm_delayRapide
    amm_timerRapide = micros();
    if (curVal > amm_curMax) amm_curMax = amm_curMax + 1;
    if (curVal < amm_curMin) amm_curMin = amm_curMin - 1;
  }
  
  // imposer que amm_curMax > amm_curMin + amplitude/10
  amm_curMax = max(amm_curMax, amm_curMin + (amm_absoluteMax - amm_absoluteMin)/10);

  // map habituel sur amm_curMin, amm_curMax
  curVal = constrain(curVal, amm_curMin+amm_hysteresisZero, amm_curMax);
  uint32_t mapped = floor(map(curVal, amm_curMin+amm_hysteresisZero, amm_curMax, amm_absoluteMin, amm_absoluteMax));
  return mapped;
}


