#include "Arduino.h"
#include "Sequence.h"

Sequence::Sequence(uint32_t siz, boolean looped) {
  _siz = _maxsiz = siz;
  _index = 0;
  _looped = looped;
  sequence = new uint8_t[_siz];
  emptySequence();
}

void Sequence::rewindSequence() {
  // rembobine la séquence
  // place le pointeur sur le premier élément
  _index = 0;
}

void Sequence::emptySequence() {
  // efface la séquence
  _index = 0;
  _siz = 0;
}

boolean Sequence::finished() {
  return (_index >= _siz);
}

uint8_t Sequence::sequenceGetNextNote() {
  uint8_t lightVal = 0;
  
  // y a-t-il vraiment une séquence à jouer ?
  if (_siz != 0 && (_index < _siz)) {
  
    // récupérer la note dans la sequence, et la jouer sur la LED
    lightVal = sequence[_index];
    // programmer la prochaine note
    _index++;
    // boucle si nécessaire
    if (_index >= _siz && _looped) _index = 0;
  }

  return lightVal;
}

// retourne true si il reste de la place
boolean Sequence::sequenceSetNextNote(uint8_t lightVal) {
  sequence[_index] = lightVal;

  // programmer la prochain enregistrement et sa date
  _index++;

  // cas spécial de durée max
  if (_index >= _maxsiz)
    return false;
  
  // bouger le curseur de fin de sequence
  _siz = _index;
  return true;
}

void Sequence::mixSequence2(uint8_t *seq2, int nbSteps2) {
  // mélange une nouvelle séquence à l'ancienne
  // à l'endroit où on jouait (_index)
  // ?? boucler si on atteint la fin de la séquence courante ? Ou ralonger ? Pour l'instant, on ralonge.
  
  for (int i = 0; i < nbSteps2 && _index + i < _maxsiz; i++) {
    // pour chaque note, fait la moyenne des deux
    byte newNote = seq2[i];
    byte oldNote = sequence[_index + i];
    int mix = (newNote + oldNote)/2;
    sequence[_index + i] = mix;
  }
  
  // mettre à jour la longueur si elle a augmenté
  _siz = max(_siz, min(_index + nbSteps2, _maxsiz));
}


void Sequence::patternTriangle(uint32_t siz) {
  // remplit la séquence avec un pattern LFO Saw
  _siz = siz;
  if (_siz >= _maxsiz) _siz = _maxsiz-1;
  for (int i = 0; i < _siz/2; i++) sequence[i] = i*255*2/_siz;
  for (int i = _siz/2; i < _siz; i++) sequence[i] = 2*255 - 2*255*i/_siz;
  rewindSequence();
}

void Sequence::patternSquare(uint32_t siz) {
  // remplit la séquence avec un pattern LFO carré
  _siz = siz;
  if (_siz >= _maxsiz) _siz = _maxsiz-1;
  for (int i = 0; i < _siz/2; i++) sequence[i] = 0;
  for (int i = _siz/2; i < _siz; i++) sequence[i] = 255;
  rewindSequence();
}

void Sequence::patternSaw(uint32_t siz) {
  // remplit la séquence avec un pattern LFO Saw
  _siz = siz;
  if (_siz >= _maxsiz) _siz = _maxsiz-1;
  for (int i = 0; i < _siz; i++) sequence[i] = i*255/_siz;
  rewindSequence();
}

void Sequence::patternFadeOut(uint32_t siz) {
  // remplit la séquence avec un pattern LFO Saw
  _siz = siz;
  if (_siz >= _maxsiz) _siz = _maxsiz-1;
  for (int i = 0; i < _siz; i++) sequence[i] = 255*(_siz-i)/_siz;
  rewindSequence();
}

void Sequence::patternDirac(uint32_t siz_on, uint32_t siz_off, uint32_t nb) {
  _siz = nb*(siz_off+siz_on);
  if (_siz >= _maxsiz) _siz = _maxsiz-1;
  for (int i = 0; i < nb; i++) {
    for (int j = 0; j < siz_on; j++) {
      int k = i*(siz_on+siz_off)+j;
      if (k >= _siz) return;
      sequence[k] = 255;
    }
    for (int j = 0; j < siz_off; j++) {
      int k = i*(siz_on+siz_off)+siz_on+j;
      if (k >= _siz) return;
      sequence[k] = 0;
    }
  }
  rewindSequence();
}


